@@ -144,6 +144,11 @@ struct dhcpcd_ctx {
 	size_t duid_len;
 	struct if_head *ifaces;
 
+	char *ctl_buf;
+	size_t ctl_buflen;
+	size_t ctl_bufpos;
+	size_t ctl_extra;
+
 	rb_tree_t routes;	/* our routes */
 #ifdef RT_FREE_ROUTE_TABLE
 	rb_tree_t froutes;	/* free routes for re-use */
@@ -151,6 +156,9 @@ struct dhcpcd_ctx {
 	size_t rt_order;	/* route order storage */
 
 	int pf_inet_fd;
+#ifdef PF_LINK
+	int pf_link_fd;
+#endif
 	void *priv;
 	int link_fd;
 #ifndef SMALL
@@ -201,6 +209,11 @@ struct dhcpcd_ctx {
 	struct ps_process_head ps_processes;	/* List of spawned processes */
 	pid_t ps_inet_pid;
 	int ps_inet_fd;		/* Network Proxy commands and data */
+	pid_t ps_control_pid;
+	int ps_control_fd;	/* Control Proxy - generic listener */
+	int ps_control_data_fd;	/* Control Proxy - data query */
+	struct fd_list *ps_control;		/* Queue for the above */
+	struct fd_list *ps_control_client;	/* Queue for the above */
 #endif
 
 #ifdef INET
