@@ -88,6 +88,10 @@
 #endif
 #endif
 
+#ifdef ELOOP_DEBUG
+#include <stdio.h>
+#endif
+
 /*
  * time_t is a signed integer of an unspecified size.
  * To adjust for time_t wrapping, we need to work the maximum signed
@@ -277,6 +281,10 @@ eloop_event_setup_fds(struct eloop *eloop)
 
 	pfd = eloop->fds;
 	TAILQ_FOREACH(e, &eloop->events, next) {
+#ifdef ELOOP_DEBUG
+		fprintf(stderr, "%s(%d) fd=%d, rcb=%p, wcb=%p\n",
+		    __func__, getpid(), e->fd, e->read_cb, e->write_cb);
+#endif
 		e->pollfd = pfd;
 		pfd->fd = e->fd;
 		pfd->events = 0;
@@ -289,6 +297,13 @@ eloop_event_setup_fds(struct eloop *eloop)
 	}
 }
 
+size_t
+eloop_event_count(const struct eloop *eloop)
+{
+
+	return eloop->nevents;
+}
+
 int
 eloop_event_add_rw(struct eloop *eloop, int fd,
     void (*read_cb)(void *), void *read_cb_arg,
@@ -326,17 +341,27 @@ eloop_event_add_rw(struct eloop *eloop, int fd,
 			e = malloc(sizeof(*e));
 			if (e == NULL)
 				return -1;
-			TAILQ_INSERT_HEAD(&eloop->events, e, next);
 		}
-		e->fd = fd;
+		TAILQ_INSERT_HEAD(&eloop->events, e, next);
 		eloop->nevents++;
+		e->fd = fd;
+		e->read_cb = read_cb;
+		e->read_cb_arg = read_cb_arg;
+		e->write_cb = write_cb;
+		e->write_cb_arg = write_cb_arg;
+		goto setup;
 	}
 
-	e->read_cb = read_cb;
-	e->read_cb_arg = read_cb_arg;
-	e->write_cb = write_cb;
-	e->write_cb_arg = write_cb_arg;
+	if (read_cb) {
+		e->read_cb = read_cb;
+		e->read_cb_arg = read_cb_arg;
+	}
+	if (write_cb) {
+		e->write_cb = write_cb;
+		e->write_cb_arg = write_cb_arg;
+	}
 
+setup:
 	eloop_event_setup_fds(eloop);
 	return 0;
 }
@@ -374,8 +399,6 @@ eloop_event_delete_write(struct eloop *eloop, int fd, int write_only)
 	}
 
 	if (write_only) {
-		if (e->write_cb == NULL)
-			return 0;
 		if (e->read_cb == NULL)
 			goto remove;
 		e->write_cb = NULL;
