@@ -31,6 +31,8 @@
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 
+#include <fcntl.h> /* Needs to be here for old Linux */
+
 #include "config.h"
 
 #include <net/if.h>
@@ -54,7 +56,6 @@
 #include <errno.h>
 #include <ifaddrs.h>
 #include <inttypes.h>
-#include <fcntl.h>
 #include <fnmatch.h>
 #include <stddef.h>
 #include <stdio.h>
@@ -107,6 +108,16 @@ if_opensockets(struct dhcpcd_ctx *ctx)
 	if (if_opensockets_os(ctx) == -1)
 		return -1;
 
+#ifdef IFLR_ACTIVE
+	ctx->pf_link_fd = xsocket(PF_LINK, SOCK_DGRAM | SOCK_CLOEXEC, 0);
+	if (ctx->pf_link_fd == -1)
+		return -1;
+#ifdef HAVE_CAPSICUM
+	if (ps_rights_limit_ioctl(ctx->pf_link_fd) == -1)
+		return -1;
+#endif
+#endif
+
 	/* We use this socket for some operations without INET. */
 	ctx->pf_inet_fd = xsocket(PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
 	if (ctx->pf_inet_fd == -1)
@@ -121,6 +132,10 @@ if_closesockets(struct dhcpcd_ctx *ctx)
 
 	if (ctx->pf_inet_fd != -1)
 		close(ctx->pf_inet_fd);
+#ifdef PF_LINK
+	if (ctx->pf_link_fd != -1)
+		close(ctx->pf_link_fd);
+#endif
 
 	if (ctx->priv) {
 		if_closesockets_os(ctx);
@@ -327,7 +342,12 @@ if_learnaddrs(struct dhcpcd_ctx *ctx, struct if_head *ifs,
 		}
 	}
 
-	freeifaddrs(*ifaddrs);
+#ifdef PRIVSEP_GETIFADDRS
+	if (IN_PRIVSEP(ctx))
+		free(*ifaddrs);
+	else
+#endif
+		freeifaddrs(*ifaddrs);
 	*ifaddrs = NULL;
 }
 
@@ -379,7 +399,6 @@ if_discover(struct dhcpcd_ctx *ctx, struct ifaddrs **ifaddrs,
 	const struct sockaddr_dl *sdl;
 #ifdef IFLR_ACTIVE
 	struct if_laddrreq iflr = { .flags = IFLR_PREFIX };
-	int link_fd;
 #endif
 #elif defined(AF_PACKET)
 	const struct sockaddr_ll *sll;
@@ -394,7 +413,7 @@ if_discover(struct dhcpcd_ctx *ctx, struct ifaddrs **ifaddrs,
 	}
 	TAILQ_INIT(ifs);
 
-#if defined(PRIVSEP) && defined(HAVE_CAPSICUM)
+#ifdef PRIVSEP_GETIFADDRS
 	if (ctx->options & DHCPCD_PRIVSEP) {
 		if (ps_root_getifaddrs(ctx, ifaddrs) == -1) {
 			logerr("ps_root_getifaddrs");
@@ -409,15 +428,6 @@ if_discover(struct dhcpcd_ctx *ctx, struct ifaddrs **ifaddrs,
 		return NULL;
 	}
 
-#ifdef IFLR_ACTIVE
-	link_fd = xsocket(PF_LINK, SOCK_DGRAM | SOCK_CLOEXEC, 0);
-	if (link_fd == -1) {
-		logerr(__func__);
-		free(ifs);
-		return NULL;
-	}
-#endif
-
 	for (ifa = *ifaddrs; ifa; ifa = ifa->ifa_next) {
 		if (ifa->ifa_addr != NULL) {
 #ifdef AF_LINK
@@ -514,7 +524,7 @@ if_discover(struct dhcpcd_ctx *ctx, struct ifaddrs **ifaddrs,
 			    MIN(ifa->ifa_addr->sa_len, sizeof(iflr.addr)));
 			iflr.flags = IFLR_PREFIX;
 			iflr.prefixlen = (unsigned int)sdl->sdl_alen * NBBY;
-			if (ioctl(link_fd, SIOCGLIFADDR, &iflr) == -1 ||
+			if (ioctl(ctx->pf_link_fd, SIOCGLIFADDR, &iflr) == -1 ||
 			    !(iflr.flags & IFLR_ACTIVE))
 			{
 				if_free(ifp);
@@ -648,9 +658,6 @@ if_discover(struct dhcpcd_ctx *ctx, struct ifaddrs **ifaddrs,
 		TAILQ_INSERT_TAIL(ifs, ifp, next);
 	}
 
-#ifdef IFLR_ACTIVE
-	close(link_fd);
-#endif
 	return ifs;
 }
 
