@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: BSD-2-Clause */
 /*
  * Privilege Separation for dhcpcd, control proxy
- * Copyright (c) 2006-2021 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2020 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -27,6 +27,7 @@
  */
 
 #include <errno.h>
+#include <signal.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -36,13 +37,17 @@
 #include "logerr.h"
 #include "privsep.h"
 
+#ifdef HAVE_CAPSICUM
+#include <sys/capsicum.h>
+#endif
+
 static int
 ps_ctl_startcb(void *arg)
 {
 	struct dhcpcd_ctx *ctx = arg;
 	sa_family_t af;
 
-	if (ctx->options & DHCPCD_MANAGER) {
+	if (ctx->options & DHCPCD_MASTER) {
 		setproctitle("[control proxy]");
 		af = AF_UNSPEC;
 	} else {
@@ -63,7 +68,7 @@ ps_ctl_startcb(void *arg)
 	ctx->ps_control_pid = getpid();
 
 	return control_start(ctx,
-	    ctx->options & DHCPCD_MANAGER ? NULL : *ctx->ifv, af);
+	    ctx->options & DHCPCD_MASTER ? NULL : *ctx->ifv, af);
 }
 
 static ssize_t
@@ -90,6 +95,19 @@ ps_ctl_recvmsg(void *arg)
 		logerr(__func__);
 }
 
+static void
+ps_ctl_signalcb(int sig, void *arg)
+{
+	struct dhcpcd_ctx *ctx = arg;
+
+	/* Ignore SIGINT, respect PS_STOP command or SIGTERM. */
+	if (sig == SIGINT)
+		return;
+
+	shutdown(ctx->ps_control_fd, SHUT_RDWR);
+	eloop_exit(ctx->eloop, sig == SIGTERM ? EXIT_SUCCESS : EXIT_FAILURE);
+}
+
 ssize_t
 ps_ctl_handleargs(struct fd_list *fd, char *data, size_t len)
 {
@@ -221,18 +239,20 @@ ps_ctl_start(struct dhcpcd_ctx *ctx)
 	int data_fd[2], listen_fd[2];
 	pid_t pid;
 
-	if (xsocketpair(AF_UNIX, SOCK_STREAM | SOCK_CXNB, 0, data_fd) == -1 ||
-	    xsocketpair(AF_UNIX, SOCK_STREAM | SOCK_CXNB, 0, listen_fd) == -1)
+	if (socketpair(AF_UNIX, SOCK_STREAM | SOCK_CXNB, 0, data_fd) == -1)
+		return -1;
+	if (socketpair(AF_UNIX, SOCK_STREAM | SOCK_CXNB, 0, listen_fd) == -1)
 		return -1;
 #ifdef PRIVSEP_RIGHTS
-	if (ps_rights_limit_fdpair(data_fd) == -1 ||
-	    ps_rights_limit_fdpair(listen_fd) == -1)
+	if (ps_rights_limit_fdpair(data_fd) == -1)
+		return -1;
+	if (ps_rights_limit_fdpair(listen_fd) == -1)
 		return -1;
 #endif
 
 	pid = ps_dostart(ctx, &ctx->ps_control_pid, &ctx->ps_control_fd,
 	    ps_ctl_recvmsg, ps_ctl_dodispatch, ctx,
-	    ps_ctl_startcb, NULL,
+	    ps_ctl_startcb, ps_ctl_signalcb,
 	    PSF_DROPPRIVS);
 
 	if (pid == -1)
@@ -263,7 +283,14 @@ ps_ctl_start(struct dhcpcd_ctx *ctx)
 	    ps_ctl_listen, ctx) == -1)
 		return -1;
 
-	ps_entersandbox("stdio inet", NULL);
+#ifdef HAVE_CAPSICUM
+	if (cap_enter() == -1 && errno != ENOSYS)
+		logerr("%s: cap_enter", __func__);
+#endif
+#ifdef HAVE_PLEDGE
+	if (pledge("stdio inet", NULL) == -1)
+		logerr("%s: pledge", __func__);
+#endif
 	return 0;
 }
 
